/*
 * $Id: cursesui.c,v 1.26 2005/04/09 05:45:03 kos Exp $
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/param.h>
#include <curses.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "cursesui.h"
#include "util.h"
#include "mach.h"
#include "ouidb.h"

#ifdef HAVE_PCAP_H
#include "pcap.h"
#endif

#ifdef HAVE_NET_BPF_H
#include "net/bpf.h"
#endif

extern char pcap_version[];

#define UI_STATUS_POS 0
#define UI_QUERY_POS 2
#define UI_HEADER_POS 3
#define UI_MAIN_POS 4

static char rotseq[] = "/-\\|";
static int disp_mode;
static int sort_mode;
static int selected_network;
static char line_info[DEF_LINELEN];

/* 
 * XXX: Order is important until this is fixed properly.
 */

uifield_t uifield[] = {
  {"SSID Name", 26, 0},
  {"MAC", 20, 0},
  {"Flg", 4, 0},
  {"Cn", 3, 0},
  {"Pkts", 7, 0},
  {"Data", 7, 0},
  {"PPS",  7, 0},
  {"", 0, 0, 0},
};

char *
status_flag_create(wnet_t *sample) {
  static char status[DEF_LINELEN];
  int i=0;
  
  status[i++] = (sample->frame.ap?'A':'H');
  if(sample->frame.wepcapable) 
    status[i++] = 'e';
  if(sample->tagged)
    status[i++] = 'T';
  if(sample->capture)
    status[i++] = 'W';
  status[i++] = '\0';
  return status;
}

char *
global_status(ssidsniff_t *s) {
  static char status[DEF_LINELEN];
  
  status[0] = '\0';
  if(s->pd_dump) 
    strncat(status, "Cap", DEF_LINELEN);

  if(s->capture == CAPTURE_DATA)
    strncat(status, "Data", DEF_LINELEN);
  else if(s->capture == CAPTURE_ALL)
    strncat(status, "All", DEF_LINELEN);

  if(s->geiger)
    strncat(status, "Snd", DEF_LINELEN);
  return status;
}

int
field_get_value(int field_id, wnet_t *sample, char *res, int resl) {
  
  switch(field_id) {
  case 0:
    snprintf(res, resl, "%-26s", string_printable(sample->frame.ssid));
    break;
  case 1:
    snprintf(res, resl, "%-20s", sample->frame.mac);
    break;
  case 2:
    snprintf(res, resl, "%-5s", status_flag_create(sample));
    break;
  case 3:
    snprintf(res, resl, "%-4d", sample->frame.channel);
    break;
  case 4:
    snprintf(res, resl, "%-7s", geek_print(sample->count.total));
    break;
  case 5:
    snprintf(res, resl, "%-7s", geek_print(sample->count.data));
    break;
  case 6:
    snprintf(res, resl, "%-7s", geek_print(sample->pps));
    break;
  }
  return 0;
}

static int
sortfunc(wnet_t *a, wnet_t *b) {
  switch(sort_mode) {
  case UI_SORT_RECV:
    return (a->serial > b->serial);
  case UI_SORT_SSID:
    return strcmp(a->frame.ssid, b->frame.ssid);
    break;
  case UI_SORT_CHANNEL:
    return (a->frame.channel < b->frame.channel);
    break;
  case UI_SORT_PACKETS:
    return (a->count.total < b->count.total);
    break;
  case UI_SORT_DATA:
    return (a->count.data < b->count.data);
    break;
  case UI_SORT_PPS:
    return (a->pps < b->pps);
    break;
  }
  return 0;
}

static int
header_show(ssidsniff_t *s) {
  static int count = 0;
  float freq;

  freq = mach_freq_get(s->device);
  move(UI_STATUS_POS, 0);
  printw("%s %c up %s  %d networks  Battery %s  %s", 
	 PACKAGE, rotseq[(count++)%4],
	 uptime(s->start), s->num_networks, mach_battery_status(),
	 global_status(s));
  move(UI_STATUS_POS+1, 0);
  printw("%s: ", s->device);
  if(freq > 0)
    printw("%.3fGHz, ", mach_freq_get(s->device)/(double)(1000000000));
  printw("%s pkts total, ", geek_print(s->globalcount.total));
  printw("%s beacons, ", geek_print(s->globalcount.beacon));
  printw("%s data, ", geek_print(s->globalcount.data));
  printw("%s unknown", geek_print(s->globalcount.martian));
  move(UI_STATUS_POS+2, 0);
  printw("%s", line_info);
  return 0;
}

static int
list_show(ssidsniff_t *s) {
  wnet_t *sample;
  int i;
  int field_id;
  int column;
  char header[DEF_LINELEN];
  char format[DEF_LINELEN];
  char value[DEF_LINELEN];

  qsort(s->networks, s->num_networks, sizeof(wnet_t), 
	(int(*)(const void *, const void *))sortfunc);
  
  // fill out current length, format string
  header[0] = '\0';
  column=1;
  for(field_id=0; uifield[field_id].size; field_id++) {
    if(uifield[field_id].size + column > COLS) {
      uifield[field_id].cursize = 0;
      break;
    }
    column += uifield[field_id].size;
    uifield[field_id].cursize = uifield[field_id].size;
  }
  
  move(UI_HEADER_POS, 0);
  attron(A_REVERSE);
  for(i=0; i<COLS; i++)
    printw(" ");
  move(UI_HEADER_POS, 1);
  for(field_id=0; uifield[field_id].cursize; field_id++) {
    snprintf(format, DEF_LINELEN, "%%-%ds", uifield[field_id].cursize);
    printw(format, uifield[field_id].name);
  }
  attroff(A_REVERSE);  
  
  for(i=0; i < MIN(s->num_networks, LINES-UI_MAIN_POS); i++) {
    sample = &s->networks[i];

    move(UI_MAIN_POS+i, 0);
    if(i == selected_network)
      printw(">");
    else
      printw(" ");   

    column = 1;
    for(field_id=0; uifield[field_id].cursize; field_id++) {
      move(UI_MAIN_POS+i, column);
      field_get_value(field_id, sample, value, DEF_LINELEN);
      printw(value);
      column += uifield[field_id].size;
    }
  }
  return 0;
}

static int
info_show(ssidsniff_t *s) {
  wnet_t *n;
  int i;

  n = &s->networks[selected_network];
  
  move(UI_HEADER_POS, 0);
  attron(A_REVERSE);
  for(i=0; i<COLS; i++)
    printw(" ");
  move(UI_HEADER_POS, 0);
  printw("Network Information for %s", n->frame.ssid);
  attroff(A_REVERSE);  
  
  move(UI_MAIN_POS, 0);
  printw("Network SSID: %s\n", n->frame.ssid);
  printw("Access point: %s\n", n->frame.ap?"Yes":"No");
  printw("MAC Address: %s\n", n->frame.mac);
  printw("BSSID Address: %s\n", n->frame.bss);
  printw("Destination Address: %s\n", n->frame.dest);
  printw("Channel: %d\n", n->frame.channel);
  printw("Frequency: %.3f\n", n->freq);
  printw("Packets per second: %u\n", n->pps);
  printw("Serial number: %d\n", n->serial);
  printw("Manufacturer: %s\n", n->manufacturer);
  printw("WEP capable: %s\n", n->frame.wepcapable?"Yes":"No");
  printw("Total packet count: %u\n", n->count.total);
  printw("Total beacon count: %u\n", n->count.beacon);
  printw("Total data count: %u\n", n->count.data);
  printw("Total unknown packet count: %u\n", n->count.martian);
  return 0;
}

static int
help_show(ssidsniff_t *s) {
  int i;

  move(UI_HEADER_POS, 0);
  attron(A_REVERSE);
  for(i=0; i<COLS; i++)
    printw(" ");
  move(UI_HEADER_POS, 0);
  printw("Help Page");
  attroff(A_REVERSE);  
  
  move(UI_MAIN_POS, 0);
  printw("\
Space  Update screen\n\
i      Information for currently selected network\n\
j      Move network selector down\n\
k      Move network selector up\n\
d      Display strings in data packets\n\
o      Select sort order: \n\
           (r)eceived, (s)sid, (c)hannel, (p)acket count, (d)ata count, \n\
           packets per seco(n)d\n\
w      Start/stop saving data packets from selected network\n\
W      Start/stop saving all data packets received\n\
A      Start/stop saving all packets including MAC layer\n\
s      Geiger mode on/off\n\
^L     Redraw screen\n\
q      Quit\n\
h      Help\n\
\n");
  printw("%s %s using libpcap %s\n", PACKAGE, VERSION, pcap_version);
  printw("Packet saving is %s, file is %s\n", s->pd_dump?"On":"Off", s->dumpfile);
  printw("Hardware address db date: %s\n", ouidb_date());
  return 0;
}

static int
strings_show(wnet_t *net) {
  int i;
  int len;

  move(UI_HEADER_POS, 0);
  attron(A_REVERSE);
  for(i=0; i<COLS; i++)
    printw(" ");
  move(UI_HEADER_POS, 0);
  printw("Data strings");
  attroff(A_REVERSE);  
  move(UI_MAIN_POS, 0);

  len = sizeof(net->strings.strings);
  for(i=0; i<len; i++)
    addch(net->strings.strings[i]);
  return 0;
}

int
ui_init(void) {  
  disp_mode = UI_MODE_NORMAL;
  sort_mode = UI_SORT_RECV;
  initscr();
  cbreak();
  noecho();
  line_info[0] = '\0';
  return 0;
}

int
ui_info(char *fmt, ...) {
  va_list vl;

  va_start(vl, fmt);
  vsnprintf(line_info, DEF_LINELEN, fmt, vl);
  va_end(vl);
  return 0;
}

int
ui_mode(ui_mode_t umode) {
  disp_mode = umode;
  return 0;
}

int
ui_finish(void) {
  endwin();
  return 0;
}

int
ui_clear(void) {
  erase();
  return 0;
}

int
ui_tag(ssidsniff_t *s, int serial) {
  wnet_t *network;
  int l;

  if(wnet_get(s, serial, (void *)&network, &l))
    return 1;
  network->tagged = !network->tagged;
  ui_select(s, UI_DIR_DOWN);
  return 0;
}

int
ui_sort(ssidsniff_t *s, ui_sort_t mode) {
  sort_mode = mode;
  return 0;
}

int
ui_select(ssidsniff_t *s, int dir) {
  int lim;

  lim = MIN(LINES-UI_MAIN_POS-1, s->num_networks-1);

  if(dir == UI_DIR_UP && selected_network > 0) {
    move(UI_MAIN_POS + selected_network, 0);
    printw(" ");
    selected_network--;
    move(UI_MAIN_POS + selected_network, 0);
    printw(">");
  } else if(dir == UI_DIR_DOWN && selected_network < lim) {
    move(UI_MAIN_POS + selected_network, 0);
    printw(" ");
    selected_network++;
    move(UI_MAIN_POS + selected_network, 0);
    printw(">");
  }
  return 0;
}

int
ui_process_events(ssidsniff_t *s) {
  int need_redraw = 0;

  switch(disp_mode) {
  case UI_MODE_NORMAL:

    switch(getch()) {
    case 014: /* ^L */
      clear();
      need_redraw = 1;
      break;
    case 'q': /* quit */
      s->running = 0;
      break;
    case 'h':
    case '?':
      disp_mode = UI_MODE_HELP;
      need_redraw = 1;
      break;
    case 'i':
      disp_mode = UI_MODE_INFO;
      need_redraw = 1;
      break;
    case ' ': /* refresh */      
      need_redraw = 1;
      break;
    case 'j': /* down */
      ui_select(s, UI_DIR_DOWN);
      break;
    case 'k': /* up */
      ui_select(s, UI_DIR_UP);
      break;
    case 'o': /* sort order */
      disp_mode = UI_MODE_SORT;
      need_redraw = 1;
      break;
    case 'w': /* capture data on this network */
      s->networks[selected_network].capture = !s->networks[selected_network].capture;
      break;
    case 'W': /* capture all data */
      if(s->capture == CAPTURE_NONE)
	s->capture = CAPTURE_DATA;
      else
	s->capture = CAPTURE_NONE;
      break;
    case 'A': /* capture all packets */
      if(s->capture == CAPTURE_NONE)
	s->capture = CAPTURE_ALL;
      else
	s->capture = CAPTURE_NONE;
      break;
    case 'd': /* strings */
      disp_mode = UI_MODE_STRINGS;
      need_redraw = 1;
      break;
    case 's': /* Sound */
      s->geiger = !s->geiger;
    }
    break;

  case UI_MODE_HELP:
  case UI_MODE_INFO:
  case UI_MODE_STRINGS:
    getch();
    disp_mode = UI_MODE_NORMAL;
    need_redraw = 1;
    break;

  case UI_MODE_SORT:
    switch(getch()) {
    case 'r': /* recv */
      ui_sort(s, UI_SORT_RECV);
      break;
    case 's': /* ssid */
      ui_sort(s, UI_SORT_SSID);
      break;
    case 'c': /* channel */
      ui_sort(s, UI_SORT_CHANNEL);
      break;
    case 'p': /* packets */
      ui_sort(s, UI_SORT_PACKETS);
      break;
    case 'd': /* data packets */
      ui_sort(s, UI_SORT_DATA);
      break;
    case 'n': /* PPS */
      ui_sort(s, UI_SORT_PPS);
      break;
    default:
      ui_sort(s, UI_SORT_RECV);
      break;
    }
    
    disp_mode = UI_MODE_NORMAL;
    need_redraw = 1;
    break;
  }

  if(need_redraw)
    ui_redraw(s);
  else
    ui_refresh(s);
  
  return 0;
}

int
ui_refresh(ssidsniff_t *s) {

  refresh();
  return 0;
}

int
ui_redraw(ssidsniff_t *s) {
  static time_t prev = 0;
  time_t now;

  ui_clear();
  switch(disp_mode) {
  case UI_MODE_NORMAL:
    header_show(s);
    list_show(s);
    break;
  case UI_MODE_INFO:
    header_show(s);  
    info_show(s);
    break;
  case UI_MODE_HELP:
    header_show(s);
    help_show(s);
    break;
  case UI_MODE_SORT:
    header_show(s);
    list_show(s);
    move(UI_QUERY_POS, 0);
    printw("Sort order? ");
    break;
  case UI_MODE_STRINGS:
    header_show(s);
    strings_show(&s->networks[selected_network]);
    break;
  }

  now = time(NULL);
  if(now - prev > 3) {
    ui_info("");
    prev = now;
  }

  refresh();
  return 0;
}
