/*
 * $Id: ssidsniff.h,v 1.24 2005/03/09 08:45:33 kos Exp $
 *
 */

#ifndef _SSIDSNIFF_H
#define _SSIDSNIFF_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <sys/time.h>
#include <pcap.h>
#include <stdio.h>
#include <unistd.h>

#include "util.h"
#include "xpaudio.h"
#include "80211.h"

#define DEF_LINELEN 1024
#define DEF_GEIGER_L 128
#define DEF_WNET_ALLOC 64
#define DEF_WRFILE "ssidsniff.cap"

typedef enum {
  CAPTURE_NONE=0, CAPTURE_DATA, CAPTURE_ALL
} capture_mode_t;

typedef struct counter_t {
  unsigned int total;
  unsigned int beacon;
  unsigned int data;
  unsigned int martian;
} counter_t;

typedef struct wnet_t {
  unsigned int serial;
  f80211_t frame;
  int tagged;
  int capture;
  char manufacturer[DEF_LINELEN];
  counter_t count;
  unsigned int pps;
  struct timeval pps_prevts;
  unsigned int pps_delta;
  unsigned int snr;
  double freq;
  strings_t strings;
} wnet_t;

typedef struct ssidsniff_t {
  char device[PCAP_ERRBUF_SIZE];
  char dumpfile[DEF_LINELEN];
  wnet_t *networks;
  int num_networks;
  int max_networks;
  int geiger;
  int delay;
  int maxcount; 
  int running;
  int capture;
  time_t start;
  xpaudio_t *xpa;
  counter_t globalcount;
  pcap_t *pd;
  int pcap_fd;
  pcap_dumper_t *pd_dump;
  pid_t hopper_pid;
} ssidsniff_t;

int wnet_clear(wnet_t *);
int wnet_exists(ssidsniff_t *, char *key);
int wnet_add(ssidsniff_t *s, char *key, void *b, unsigned int l);
int wnet_get(ssidsniff_t *s, int which, void **b, unsigned int *l);
int field_get_value(int field_id, wnet_t *sample, char *res, int resl);

#endif /* _SSIDSNIFF_H */
