/*
 * $Id: util.c,v 1.5 2005/03/06 03:45:46 kos Exp $
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <time.h>
#include <string.h>
#include <stdio.h>

#include "util.h"

void
tokenize(char *usrc, char **tokens, int *ntokens) {
  int i;
  static char src[DEF_LINELEN];
  
  strncpy(src, usrc, DEF_LINELEN);
  for(i=1,tokens[0] = strtok(src, " "); (tokens[i]=strtok(NULL, " ")); i++)
    ;
  *ntokens = i;
}

unsigned char *
geek_print(unsigned int n) {
  static char buf[DEF_LINELEN];
  
  if(n < 1024)
    snprintf(buf, DEF_LINELEN, "%d", n);
  else if(n<1048576)
    snprintf(buf, DEF_LINELEN, "%.1fK", (float)n/(float)1024);
  else if(n<1073741824)
    snprintf(buf, DEF_LINELEN, "%.1fM", (float)n/(float)1048576);
  else
    snprintf(buf, DEF_LINELEN, "%.1fG", (float)n/(float)1073741824);
  return buf;
}

unsigned char *
uptime(time_t start) {
  static char buf[DEF_LINELEN];
  time_t uptime;
  struct tm *tm;

  uptime = time(NULL) - start;
  tm = gmtime(&uptime);
  if(tm->tm_yday)
    snprintf(buf, DEF_LINELEN, "%d days, %02d:%02d:%02d", 
	     tm->tm_yday, tm->tm_hour, tm->tm_min, tm->tm_sec);
  else
    snprintf(buf, DEF_LINELEN, "%02d:%02d:%02d", 
	     tm->tm_hour, tm->tm_min, tm->tm_sec);
  return buf;
}

unsigned char *
mac2ascii(unsigned char *p) {
  static char out[DEF_LINELEN];
  
  snprintf(out, DEF_LINELEN, "%02x:%02x:%02x:%02x:%02x:%02x", 
	   p[0], p[1], p[2], p[3], p[4], p[5]);
  return out;
}

unsigned char *
hex2ascii(unsigned char *p, int l) {
  int i;
  static char out[DEF_LINELEN];
  char temp[DEF_LINELEN];

  out[0] = '\0';
  for(i=0; i<l; i++) {
    snprintf(temp, DEF_LINELEN, "%02x", p[i]);
    strncat(out, temp, DEF_LINELEN);
  }
  return out;
}

unsigned char *
string_printable(unsigned char *in) {
  static char out[DEF_LINELEN];
  char octal[DEF_LINELEN];
  char *ret;

  memset(out, '\0', DEF_LINELEN);
  ret = out;

  while(*in) {
    if(*in >= 32 && *in <= 127) {
      *ret++ = *in;
    } else if(*in == '\\') {
      *ret++ = '\\';
      *ret++ = '\\';
    } else if(*in == '\a') {
      *ret++ = '\\';
      *ret++ = 'a';
    } else if(*in == '\b') {
      *ret++ = '\\';
      *ret++ = 'b';
    } else if(*in == '\n') {
      *ret++ = '\\';
      *ret++ = 'n';
    } else if(*in == '\r') {
      *ret++ = '\\';
      *ret++ = 'r';
    } else {
      snprintf(octal, DEF_LINELEN, "\\%03o", *in);
      strncat(ret, octal, strlen(octal));
      ret += strlen(octal);
    }
    in++;
  }
  return out;
}


int
strings_clear(strings_t *s) {
  memset(s->strings, 0x20, sizeof(s->strings));
  s->start = 0;
  return 0;
}

int
strings_add(strings_t *s, unsigned char *p, int l) {
  char temp[DEF_LINELEN];
  int i;
  int j, k;

  j = 0;
  for(i=0; i<l; i++) {

    if(j == DEF_LINELEN) {
      for(k=0; k<j; k++) {
	s->strings[s->start] = temp[k];
	s->start = (s->start + 1) % sizeof(s->strings); 
      }
      j=0;
    }

    if(ISSTRING(p[i]))
      temp[j++] = p[i];
    else {      
      if(j >= STRINGS_LEN) {
	for(k=0; k<j; k++) {
	  s->strings[s->start] = temp[k];
	  s->start = (s->start + 1) % sizeof(s->strings);
	}
	s->strings[s->start] = ' ';
	s->start = (s->start + 1) % sizeof(s->strings);
      }
      j = 0;
    }
  }
  return 0;
}
